package org.processmining.performancespectrumintegration.algorithms.miner;

import com.google.common.collect.Maps;
import org.processmining.performancespectrumintegration.algorithms.segments.OrderedSegmentMeasurement;
import org.processmining.performancespectrumintegration.algorithms.segments.Segment;
import org.processmining.performancespectrumintegration.models.settings.Filter;

import java.util.Iterator;
import java.util.Map;

public class SegmentData implements Iterable<Map.Entry<Segment, OrderedSegmentMeasurement>> {

    private final Map<Segment, OrderedSegmentMeasurement> internal;

    public SegmentData() {
        internal = Maps.newHashMap();
    }

    public void add(Segment segment, OrderedSegmentMeasurement orderedSegmentMeasurement) {
        if (!internal.containsKey(segment))
            internal.put(segment, new OrderedSegmentMeasurement());
        internal.get(segment).merge(orderedSegmentMeasurement);
    }

    public OrderedSegmentMeasurement get(Segment segment) {
        return internal.get(segment);
    }

    public void restrictCases(Filter<Integer> filter) {
        internal.forEach((s, sm) -> sm.restrictCases(filter));
    }

    public void merge(SegmentData other) {
        assert other != null;
        for (Map.Entry<Segment, OrderedSegmentMeasurement> entry : other.internal.entrySet()) {
            internal.compute(entry.getKey(), (s, v) -> {
                if (v == null)
                    v = new OrderedSegmentMeasurement();
                v.merge(entry.getValue());
                return v;
            });
        }
    }

    @Override
    public Iterator<Map.Entry<Segment, OrderedSegmentMeasurement>> iterator() {
        return internal.entrySet().iterator();
    }
}
