package org.processmining.performancespectrumintegration.algorithms.petrinet;

import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;

import java.util.*;
import java.util.function.BiConsumer;
import java.util.function.Function;

public class ExtendedMarking<T extends Comparable<T>> {

    private final BiConsumer<Deque<T>, T> addAction;
    private final Function<Deque<T>, T> peekFunction;
    private final Function<Deque<T>, T> removeFunction;
    private final Map<Place, Deque<T>> tokens;

    public ExtendedMarking() {
        // default queue model
        tokens = Maps.newHashMap();
        addAction = Deque::addLast;
        peekFunction = Deque::peekFirst;
        removeFunction = Deque::removeFirst;
    }


    public void addTo(Place place, T source) {
        tokens.compute(place, (key, coll) -> {
            if (coll == null) coll = new LinkedList<>();
            addAction.accept(coll, source);
            return coll;
        });
    }

    public void addTo(Set<Place> places, T source) {
        places.forEach(p -> tokens.compute(p, (key, coll) -> {
            if (coll == null) coll = new LinkedList<>();
            addAction.accept(coll, source);
            return coll;
        }));
    }

    public SortedSet<T> peekFrom(Set<Place> places) {
        SortedSet<T> result = Sets.newTreeSet();
        for (Place p : places) {
            if (tokens.containsKey(p))
                result.add(peekFunction.apply(tokens.get(p)));
        }
        return result;
    }

    public Set<T> removeFrom(Set<Place> places) {
        Set<T> result = Sets.newHashSet();
        for (Place p : places) {
            if (tokens.containsKey(p)) {
                Deque<T> deque = tokens.get(p);
                result.add(removeFunction.apply(deque));
                if (deque.isEmpty())
                    tokens.remove(p);
            }

        }
        return result;
    }

}
