package org.processmining.performancespectrumintegration.algorithms.petrinet;

import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

public class GraphTools {

    public static Map<Transition, AdjPlaces> getAdjPlacesMap(Petrinet net) {
        Map<Transition, AdjPlaces> adjPlacesMap = Maps.newHashMapWithExpectedSize(net.getTransitions().size());
        for (final Transition transition : net.getTransitions()) {
            Set<Place> pre = net.getInEdges(transition).stream().map(e -> (Place) e.getSource()).collect(Collectors.toSet());
            Set<Place> post = net.getOutEdges(transition).stream().map(e -> (Place) e.getTarget()).collect(Collectors.toSet());
            AdjPlaces adjPlaces = new AdjPlaces(pre, post);
            adjPlacesMap.put(transition, adjPlaces);
        }
        return adjPlacesMap;
    }

    public static Set<Transition> getIngoingTransitions(Petrinet petrinet, Place place) {
        return petrinet.getInEdges(place).stream().map(e -> (Transition) e.getSource()).collect(Collectors.toSet());
    }

    public static Set<Transition> getOutgoingTransitions(Petrinet petrinet, Place place) {
        return petrinet.getOutEdges(place).stream().map(e -> (Transition) e.getTarget()).collect(Collectors.toSet());
    }

    public static Quadruple<Set<Place>, Set<Transition>, Set<PetrinetEdge<Transition, Place>>, Set<PetrinetEdge<Place, Transition>>> crawlBetween(final Petrinet petrinet, final Set<Transition> from, final Set<Transition> to) {
        Set<Place> collectedPlaces = Sets.newHashSet();
        Set<Transition> collectedTransitions = Sets.newHashSet();
        Set<PetrinetEdge<Transition, Place>> collectedTransitonEdges = Sets.newHashSet();
        Set<PetrinetEdge<Place, Transition>> collectedPlaceEdges = Sets.newHashSet();

        Set<Transition> current = from;
        int counter = 0;
        while (!current.isEmpty()) {
            if (counter >= 100) {
                System.out.println("Search in between transitions had to be cancelled");
                return null;
            }

            Set<Place> places = transitionsPostSet(petrinet, current, collectedTransitonEdges);
            collectedPlaces.addAll(places);
            Set<Transition> transitions = placesPostSet(petrinet, places, collectedPlaceEdges);
            collectedTransitions.addAll(transitions);

            transitions.removeAll(current);
            transitions.removeAll(to);
            current = transitions;
            counter++;
        }


        return new Quadruple<>(collectedPlaces, collectedTransitions, collectedTransitonEdges, collectedPlaceEdges);
    }

    private static Set<Place> transitionsPostSet(Petrinet petrinet, Set<Transition> from, Set<PetrinetEdge<Transition, Place>> collectedEdges) {
        Set<Place> set = new HashSet<>();
        for (Transition n : from) {
            for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> petrinetEdge : petrinet.getOutEdges(n)) {
                collectedEdges.add((PetrinetEdge<Transition, Place>) petrinetEdge);
                PetrinetNode target = petrinetEdge.getTarget();
                Place place = (Place) target;
                set.add(place);
            }
        }
        return set;
    }

    private static Set<Transition> placesPostSet(Petrinet petrinet, Set<Place> from, Set<PetrinetEdge<Place, Transition>> collectedEdges) {
        Set<Transition> set = new HashSet<>();
        for (Place n : from) {
            for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> petrinetEdge : petrinet.getOutEdges(n)) {
                collectedEdges.add((PetrinetEdge<Place, Transition>) petrinetEdge);
                PetrinetNode target = petrinetEdge.getTarget();
                Transition transition = (Transition) target;
                set.add(transition);
            }
        }
        return set;
    }

    public static class Quadruple<A, B, C, D> {
        public A _1;
        public B _2;
        public C _3;
        public D _4;

        public Quadruple(A _1, B _2, C _3, D _4) {
            this._1 = _1;
            this._2 = _2;
            this._3 = _3;
            this._4 = _4;
        }
    }

}
