package org.processmining.performancespectrumintegration.algorithms.segments;

import com.google.common.base.Objects;
import com.google.common.collect.ImmutableSet;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

import java.util.Collection;
import java.util.stream.Collectors;

public class MultiSegment implements Segment {

    private final ImmutableSet<Transition> from, to;

    public MultiSegment(Collection<Transition> from, Collection<Transition> to) {
        assert !from.isEmpty() && !to.isEmpty();
        this.from = ImmutableSet.copyOf(from);
        this.to = ImmutableSet.copyOf(to);
    }

    public MultiSegment(ImmutableSet<Transition> from, ImmutableSet<Transition> to) {
        this.from = from;
        this.to = to;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        MultiSegment that = (MultiSegment) o;
        return Objects.equal(from, that.from) &&
                Objects.equal(to, that.to);
    }

    @Override
    public int hashCode() {
        return Objects.hashCode(from, to);
    }

    @Override
    public String toString() {
        return "[" + from.stream().map(Object::toString).collect(Collectors.joining(",")) + "]:[" + to.stream().map(Object::toString).collect(Collectors.joining(",")) + "]";
    }

    @Override
    public ImmutableSet<Transition> getFrom() {
        return from;
    }

    @Override
    public ImmutableSet<Transition> getTo() {
        return to;
    }
}
