package org.processmining.performancespectrumintegration.algorithms.segments;

import org.joda.time.DateTime;
import org.joda.time.Duration;

public class SegmentOccurrence implements Comparable<SegmentOccurrence> {

    private final DateTime startTime;
    private final Duration duration;
    private final Integer caseId;
    // class etc. event attributes

    public SegmentOccurrence(DateTime startTime, Duration duration, Integer caseId) {
        this.startTime = startTime;
        this.duration = duration;
        this.caseId = caseId;
    }

    public SegmentOccurrence(DateTime startTime, DateTime endTime, Integer caseId) {
        this.startTime = startTime;
        this.caseId = caseId;
        this.duration = new Duration(startTime, endTime);
    }

    public static SegmentOccurrence artificial(DateTime dateTime) {
        return new SegmentOccurrence(dateTime, dateTime, 0);
    }

    public DateTime getStartTime() {
        return startTime;
    }

    public Duration getDuration() {
        return duration;
    }

    @Override
    public String toString() {
        return "SegmentOccurrence{" +
                "startTime=" + startTime +
                ", duration=" + duration +
                '}';
    }

    public DateTime getEndTime() {
        return startTime.plus(duration);
    }

    @Override
    public int compareTo(SegmentOccurrence o) {
        return startTime.compareTo(o.getStartTime());
    }

    public Integer getCaseId() {
        return caseId;
    }
}
