package org.processmining.performancespectrumintegration.models;

public class MeasurementReliability {

    private int normalOccurrences;
    private int uncountedOccurrences;
    private int cases;

    public MeasurementReliability() {
    }

    public MeasurementReliability(int normalOccurrences, int uncountedOccurrences) {
        this.normalOccurrences = normalOccurrences;
        this.uncountedOccurrences = uncountedOccurrences;
    }

    public void incNormalOccurrences(int increase) {
        normalOccurrences += increase;
    }

    public void incUncountedOccurrences(int increase) {
        uncountedOccurrences += increase;
    }

    public int getUncountedOccurrences() {
        return uncountedOccurrences;
    }

    public void setUncountedOccurrences(int uncountedOccurrences) {
        this.uncountedOccurrences = uncountedOccurrences;
    }

    public int getTotalOccurrences() {
        return normalOccurrences + uncountedOccurrences;
    }

    public int getNormalOccurrences() {
        return normalOccurrences;
    }

    public void setNormalOccurrences(int normalOccurrences) {
        this.normalOccurrences = normalOccurrences;
    }

    public void merge(MeasurementReliability other) {
        assert other != null;
        normalOccurrences += other.normalOccurrences;
        uncountedOccurrences += other.uncountedOccurrences;
        cases += other.cases;
    }

    @Override
    public String toString() {
        return normalOccurrences + "/" + (normalOccurrences + uncountedOccurrences) + " " + toShortString() + " over " + cases + " distinct cases";
    }

    public String toShortString() {
        return "[" + String.format("%.2f%%", 100 * normalOccurrences / (double) (normalOccurrences + uncountedOccurrences)) + "]";
    }

    public boolean isUseless() {
        return normalOccurrences == 0;
    }

    public void incCaseCounter(int increase) {
        cases += increase;
    }

    public int getNumberOfCases() {
        return cases;
    }
}
