package org.processmining.performancespectrumintegration.models;

import org.joda.time.DateTime;
import org.joda.time.Duration;
import org.processmining.performancespectrumintegration.algorithms.miner.SegmentData;
import org.processmining.performancespectrumintegration.algorithms.segments.OrderedSegmentMeasurement;
import org.processmining.performancespectrumintegration.algorithms.segments.Segment;
import org.processmining.performancespectrumintegration.algorithms.segments.SegmentOccurrence;
import org.processmining.performancespectrumintegration.models.settings.CaseIdClassification;
import org.processmining.performancespectrumintegration.util.PrettyPrintable;
import org.processmining.performancespectrumintegration.util.Utils;

import java.util.List;
import java.util.Set;

public class MyDataSource implements DataControllable, PrettyPrintable {

    private CaseIdClassification classification;
    private DateTime earliestTime;
    private DateTime latestTime;
    private Duration processDuration;
    private SegmentData segmentData;
    private Set<Segment> selectedSegments;
    private List<Segment> segmentOrder;
    private TimeWindowData timeWindowData;
    private int totalCases, filteredCases;

    public TimeWindowData getTimeWindowData() {
        return timeWindowData;
    }

    public void setTimeWindowData(TimeWindowData timeWindowData) {
        this.timeWindowData = timeWindowData;
    }

    @Override
    public void setCaseNumbers(int total, int filtered) {
        totalCases = total;
        filteredCases = filtered;
    }

    @Override
    public boolean isComplete() {
        return Utils.allNonNull(classification, earliestTime, latestTime, segmentData, selectedSegments, segmentOrder, timeWindowData);
    }

    public void setSegmentData(SegmentData segmentData) {
        this.segmentData = segmentData;
    }

    public Duration getProcessDuration() {
        return processDuration;
    }

    public MeasurementReliability getSegmentReliability(Segment segment) {
        return segmentData.get(segment).getMeasurementReliability();
    }

    public Set<Segment> getSelectedSegments() {
        return selectedSegments;
    }

    @Override
    public void setSegments(Set<Segment> selectedSegments) {
        this.selectedSegments = selectedSegments;
    }

    public OrderedSegmentMeasurement getSegmentMeasurement(Segment segment) {
        return segmentData.get(segment);
    }

    public Integer getClass(Integer caseId) {
        return classification.getClass(caseId);
    }

    public CaseIdClassification getClassification() {
        return classification;
    }

    public void setClassification(CaseIdClassification classification) {
        this.classification = classification;
    }

    @Override
    public void setProcessTimes(DateTime earliestTime, DateTime latestTime) {
        this.earliestTime = earliestTime;
        this.latestTime = latestTime;
        if (earliestTime != null && latestTime != null)
            this.processDuration = new Duration(earliestTime, latestTime);
    }

    public DateTime getProcessStart() {
        return earliestTime;
    }

    public DateTime getProcessEnd() {
        return latestTime;
    }

    @Override
    public String toPrettyString() {
        StringBuilder sb = new StringBuilder();

        for (Segment segment : selectedSegments) {
            OrderedSegmentMeasurement sm = segmentData.get(segment);

            sb.append(segment.toString()).append(sm.getMeasurementReliability()).append("\n");

            for (SegmentOccurrence so : sm.getOccurrences()) {
                Integer caseId = so.getCaseId();

                sb.append("\t").append("Case id: ").append(caseId).append("\t").append("Class: ")
                        .append(classification.getClass(caseId)).append("\t").append(so.getStartTime())
                        .append("::").append(so.getDuration()).append("\n");
            }

        }

        return sb.toString();
    }

    public List<Segment> getSegmentOrder() {
        return segmentOrder;
    }

    @Override
    public void setSegmentOrder(List<Segment> segmentOrder) {
        this.segmentOrder = segmentOrder;
    }

    public int getTotalCases() {
        return totalCases;
    }

    public int getFilteredCases() {
        return filteredCases;
    }
}
