package org.processmining.performancespectrumintegration.models.settings;

import org.joda.time.Period;
import org.processmining.performancespectrumintegration.util.ToStrings;

import java.io.Serializable;
import java.util.Objects;

public class AggregationIntervalSettings implements Serializable {

    private IntervalPresetType presetType;
    private boolean normalizeProcessStart;
    private Period period;

    private AggregationIntervalSettings() {
    }

    public AggregationIntervalSettings(IntervalPresetType presetType, boolean normalizeProcessStart) {
        this.normalizeProcessStart = normalizeProcessStart;
        this.period = presetType.getPeriod();
        this.presetType = presetType;
    }

    public AggregationIntervalSettings(Period period) {
        this.period = period.normalizedStandard();
        this.normalizeProcessStart = false;
        this.presetType = IntervalPresetType.Custom;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        AggregationIntervalSettings that = (AggregationIntervalSettings) o;
        return normalizeProcessStart == that.normalizeProcessStart &&
                presetType == that.presetType &&
                Objects.equals(period, that.period);
    }

    @Override
    public int hashCode() {
        return Objects.hash(presetType, normalizeProcessStart, period);
    }

    public IntervalPresetType getPresetType() {
        return presetType;
    }

    public boolean isNormalizeProcessStart() {
        return normalizeProcessStart;
    }

    public Period getPeriod() {
        return period;
    }

    @Override
    public String toString() {
        return ToStrings.print(period) + (presetType != IntervalPresetType.Custom ? (normalizeProcessStart ? " (normalized)" : "") : " (custom)");
    }
}
