package org.processmining.performancespectrumintegration.models.settings;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;

import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

public class CaseIdFilterSettings extends FilterSettings {

    private final boolean inverted;
    private final Set<Integer> caseIds;

    @JsonCreator
    public CaseIdFilterSettings(@JsonProperty("caseIds") Set<Integer> caseIds, @JsonProperty("inverted") boolean inverted) {
        super(FilterType.CaseIds);
        this.caseIds = caseIds;
        this.inverted = inverted;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        if (!super.equals(o)) return false;
        CaseIdFilterSettings that = (CaseIdFilterSettings) o;
        return inverted == that.inverted &&
                Objects.equals(caseIds, that.caseIds);
    }

    @Override
    public int hashCode() {
        return Objects.hash(super.hashCode(), inverted, caseIds);
    }

    public Set<Integer> getCaseIds() {
        return caseIds;
    }

    @JsonIgnore
    public String getIdListString() {
        return caseIds.stream().map(Objects::toString).collect(Collectors.joining(","));
    }

    @Override
    public String toString() {
        return "ids" + (inverted ? "!=" : "=") + caseIds.toString();
    }

    public boolean isInverted() {
        return inverted;
    }
}
