package org.processmining.performancespectrumintegration.models.settings;

import com.google.common.collect.Sets;
import org.processmining.performancespectrumintegration.algorithms.segments.Segment;

import java.util.Objects;
import java.util.Set;

public class DataSettings {

    private SegmentOrderSettings segmentOrderSettings;
    private Set<Segment> selectedSegments;
    private FilterSettings filterSettings;
    private ClassificationSettings classificationSettings;
    private AggregationIntervalSettings aggregationIntervalSettings;

    public DataSettings() {

    }

    public static DataSettings defaultSettings() {
        DataSettings dataSettings = new DataSettings();
        dataSettings.setAggregationIntervalSettings(new AggregationIntervalSettings(IntervalPresetType.Month, true));
        dataSettings.setSelectedSegments(Sets.newHashSetWithExpectedSize(0));
        dataSettings.setFilterSettings(FilterSettings.NONE);
        dataSettings.setClassificationSettings(ClassificationSettings.NONE);
        dataSettings.setSegmentOrderSettings(SegmentOrderSettings.HEURISTIC);
        return dataSettings;
    }

    @Override
    public String toString() {
        return "DataSettings{" +
                "segmentOrderSettings=" + segmentOrderSettings +
                ", selectedSegments=" + selectedSegments +
                ", filterSettings=" + filterSettings +
                ", classificationSettings=" + classificationSettings +
                ", aggregationIntervalSettings=" + aggregationIntervalSettings +
                '}';
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        DataSettings that = (DataSettings) o;
        return Objects.equals(segmentOrderSettings, that.segmentOrderSettings) &&
                Objects.equals(selectedSegments, that.selectedSegments) &&
                Objects.equals(filterSettings, that.filterSettings) &&
                Objects.equals(classificationSettings, that.classificationSettings) &&
                Objects.equals(aggregationIntervalSettings, that.aggregationIntervalSettings);
    }

    @Override
    public int hashCode() {
        return Objects.hash(segmentOrderSettings, selectedSegments, filterSettings, classificationSettings, aggregationIntervalSettings);
    }

    public Set<Segment> getSelectedSegments() {
        return selectedSegments;
    }

    public void setSelectedSegments(Set<Segment> selectedSegments) {
        this.selectedSegments = selectedSegments;
    }

    public FilterSettings getFilterSettings() {
        return filterSettings;
    }

    public void setFilterSettings(FilterSettings filterSettings) {
        this.filterSettings = filterSettings;
    }

    public ClassificationSettings getClassificationSettings() {
        return classificationSettings;
    }

    public void setClassificationSettings(ClassificationSettings classificationSettings) {
        this.classificationSettings = classificationSettings;
    }

    public SegmentOrderSettings getSegmentOrderSettings() {
        return segmentOrderSettings;
    }

    public void setSegmentOrderSettings(SegmentOrderSettings segmentOrderSettings) {
        this.segmentOrderSettings = segmentOrderSettings;
    }

    public AggregationIntervalSettings getAggregationIntervalSettings() {
        return aggregationIntervalSettings;
    }

    public void setAggregationIntervalSettings(AggregationIntervalSettings aggregationIntervalSettings) {
        this.aggregationIntervalSettings = aggregationIntervalSettings;
    }

    public boolean isAnySet() {
        return aggregationIntervalSettings != null || selectedSegments != null || filterSettings != null || classificationSettings != null || segmentOrderSettings != null;
    }

    public boolean isAllSet() {
        return aggregationIntervalSettings != null && selectedSegments != null && filterSettings != null && classificationSettings != null && segmentOrderSettings != null;
    }

    public boolean isOnlySegmentOrderingSet() {
        return aggregationIntervalSettings == null && selectedSegments == null && filterSettings == null && classificationSettings == null && segmentOrderSettings != null;
    }

    public void apply(DataSettings delta) {
        if (delta.getAggregationIntervalSettings() != null)
            setAggregationIntervalSettings(delta.getAggregationIntervalSettings());
        if (delta.getFilterSettings() != null)
            setFilterSettings(delta.getFilterSettings());
        if (delta.getClassificationSettings() != null)
            setClassificationSettings(delta.getClassificationSettings());
        if (delta.getSelectedSegments() != null)
            setSelectedSegments(delta.getSelectedSegments());
        if (delta.getSegmentOrderSettings() != null)
            setSegmentOrderSettings(delta.getSegmentOrderSettings());
    }

    public DataSettings copy() {
        DataSettings copy = new DataSettings();
        copy.setAggregationIntervalSettings(aggregationIntervalSettings);
        copy.setSelectedSegments(selectedSegments);
        copy.setFilterSettings(filterSettings);
        copy.setClassificationSettings(classificationSettings);
        copy.setSegmentOrderSettings(segmentOrderSettings);
        return copy;
    }

}
