package org.processmining.performancespectrumintegration.models.settings;

import java.util.Collection;
import java.util.Set;
import java.util.stream.Collectors;

public interface Filter<T> {

    boolean isIncluded(T i);

    default Filter<T> invert() {
        return Filter.this::isNotIncluded;
    }

    default boolean isNotIncluded(T i) {
        return !isIncluded(i);
    }

    default Filter<T> intersection(final Collection<T> collection) {
        return new Filter<T>() {
            @Override
            public boolean isIncluded(T i) {
                return Filter.this.isIncluded(i) && collection.contains(i);
            }
        };
    }

    default Filter<T> intersection(final Filter<T> other) {
        return new Filter<T>() {
            @Override
            public boolean isIncluded(T i) {
                return Filter.this.isIncluded(i) && other.isIncluded(i);
            }
        };
    }

    default <K extends Collection<T>> void applyFilter(K others) {
        others.removeIf(this::isNotIncluded);
    }

    default <K extends Collection<T>> Set<T> applyFilterToImmutable(K collection) {
        return collection.stream().filter(this::isIncluded).collect(Collectors.toSet());
    }

    default <K extends Collection<T>> long countIntersection(K collection) {
        return collection.stream().filter(this::isIncluded).count();
    }

    default Filter<T> difference(final Collection<T> collection) {
        return new Filter<T>() {
            @Override
            public boolean isIncluded(T i) {
                return Filter.this.isIncluded(i) && !collection.contains(i);
            }
        };
    }

    default Filter<T> difference(final Filter<T> other) {
        return new Filter<T>() {
            @Override
            public boolean isIncluded(T i) {
                return Filter.this.isIncluded(i) && other.isNotIncluded(i);
            }
        };
    }

    default long numberOfIncluded(Collection<T> collection) {
        return collection.stream().filter(this::isIncluded).count();
    }

}
