package org.processmining.performancespectrumintegration.models.settings;

import org.joda.time.Duration;
import org.joda.time.Instant;
import org.joda.time.Period;

import java.util.Arrays;

public enum IntervalPresetType {
    Year(Period.years(1)), Quarter(Period.months(3)),
    Month(Period.months(1)), Week(Period.weeks(1)),
    Day(Period.days(1)), Hour(Period.hours(1)),
    Minute(Period.minutes(1)), Custom(null);

    private final Period period;

    IntervalPresetType(Period period) {
        this.period = period;
    }

    public static IntervalPresetType[] valuesShorterThan(Period period) {
        Duration d = period.toDurationFrom(new Instant(0));
        return Arrays.stream(values()).filter(it -> it.getPeriod() != null && it.getPeriod().toDurationFrom(new Instant(0)).isShorterThan(d)).toArray(IntervalPresetType[]::new);
    }

    public Period getPeriod() {
        return period;
    }
}
