package org.processmining.performancespectrumintegration.models.settings;

import com.google.common.collect.Maps;

import java.util.Map;
import java.util.Set;
import java.util.SortedMap;

public class MapBasedClassification implements CaseIdClassification {

    private final Map<Integer, Integer> internal; // case id -> class id
    private final SortedMap<Integer, String> legend; // class id -> string
    private final int numberOfClasses;
    private SortedMap<Integer, Integer> classSizes; // class id -> count

    public MapBasedClassification(Map<Integer, Integer> classification, SortedMap<Integer, String> legend) {
        this.internal = classification;
        this.legend = legend;
        this.numberOfClasses = legend.size();
    }

    private void computeClassSizes() {
        classSizes = Maps.newTreeMap();
        for (Integer classId : legend.keySet()) {
            classSizes.put(classId, 0);
        }
        for (Map.Entry<Integer, Integer> entry : internal.entrySet()) {
            classSizes.put(entry.getValue(), classSizes.get(entry.getValue()) + 1);
        }
    }

    @Override
    public Integer getClass(Integer caseId) {
        return internal.get(caseId);
    }

    @Override
    public SortedMap<Integer, String> getLegend() {
        return legend;
    }

    @Override
    public int getDefinitionSpaceSize() {
        return internal.size();
    }

    public Set<Integer> getClasses() {
        return legend.keySet();
    }

    @Override
    public SortedMap<Integer, Integer> getClassSizes() {
        if (classSizes == null) computeClassSizes();
        return classSizes;
    }

    @Override
    public int getNumberOfClasses() {
        return numberOfClasses;
    }

}
