package org.processmining.performancespectrumintegration.models.settings;

import java.util.Objects;

public class PercentageFilterSettings extends FilterSettings {

    private final boolean inverted;
    private final double percentage;

    public PercentageFilterSettings(FilterType type, double percentage, boolean inverted) {
        super(type);
        this.inverted = inverted;
        this.percentage = percentage;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        if (!super.equals(o)) return false;
        PercentageFilterSettings that = (PercentageFilterSettings) o;
        return inverted == that.inverted &&
                Double.compare(that.percentage, percentage) == 0;
    }

    @Override
    public int hashCode() {
        return Objects.hash(super.hashCode(), inverted, percentage);
    }

    public double getPercentage() {
        return percentage;
    }

    @Override
    public String toString() {
        return (inverted ? "bottom " : "top ") + String.format("%.2f%%", 100 * percentage);
    }

    public boolean isInverted() {
        return inverted;
    }
}
