package org.processmining.performancespectrumintegration.mypsm;

import org.joda.time.Period;
import org.processmining.performancespectrumintegration.models.MyDataSource;
import org.processmining.performancespectrumintegration.models.TimeWindowData;
import org.processmining.performancespectrumintegration.models.settings.DataSettings;
import org.processmining.performancespectrumintegration.mypsm.segmentpanel.SegmentDataPanel;
import org.processmining.performancespectrumintegration.mypsm.update.PSMUpdateListener;
import org.processmining.performancespectrumintegration.mypsm.update.PSMUpdateManager;
import org.processmining.performancespectrumintegration.mypsm.update.PSMVisualSettings;

import javax.swing.*;
import java.awt.*;

public class BetterPSMTimeScale extends SegmentDataPanel implements PSMUpdateListener {
    public static final Integer BIG_TICK_ANNOTATION_PADDING = 5;
    public static final long MAX_INBETWEEN_TICKS = 100;
    private static final int BIG_TICK_HEIGHT = 10;
    private static final int INBETWEEN_TICK_HEIGHT = 4;
    private static final int MID_LINE = 30;

    public BetterPSMTimeScale(PSMUpdateManager psmUpdateManager) {
        super(psmUpdateManager);
        setMinimumSize(new Dimension(300, MID_LINE * 2));
        setPreferredSize(new Dimension(300, MID_LINE * 2));

        psmUpdateManager.addListener(this);
    }

    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);

        Graphics2D graphics2D = (Graphics2D) g;

        int width = getWidth();

        BasicStroke thicker = new BasicStroke(2);
        BasicStroke thinner = new BasicStroke(1);

        graphics2D.setStroke(thicker);
        graphics2D.drawLine(0, MID_LINE, width, MID_LINE);

        PSMVisualSettings visualSettings = psmUpdateManager.getVisualSettings();
        MyDataSource dataSource = psmUpdateManager.getDataSource();

        if (!dataSource.isComplete()) {
            return;
        }

        TimeWindowData timeWindowData = dataSource.getTimeWindowData();

        ViewWindowTools.WindowParams viewWindow = ViewWindowTools.getWindow(visualSettings, dataSource.getProcessStart(), dataSource.getProcessDuration(), getWidth());

        Period inbetweenTickSize = visualSettings.getInbetweenTick().getPeriod();

        Color old = graphics2D.getColor();
        Color back = accent;

        iterateTicks(graphics2D.getFontMetrics(), timeWindowData, viewWindow, inbetweenTickSize, (tickPos, nextTickPos) -> {
            graphics2D.setStroke(thinner);
            graphics2D.setColor(back);
            graphics2D.fillRect(tickPos, MID_LINE - INBETWEEN_TICK_HEIGHT / 2, nextTickPos - tickPos, INBETWEEN_TICK_HEIGHT);
            graphics2D.setStroke(thicker);
            graphics2D.setColor(old);
        }, (isBelow, tickPos) -> {
            if (isBelow) graphics2D.drawLine(tickPos, MID_LINE, tickPos, MID_LINE + BIG_TICK_HEIGHT);
            else graphics2D.drawLine(tickPos, MID_LINE - BIG_TICK_HEIGHT, tickPos, MID_LINE);
        }, (isBelow, annotationPos, print) -> {
            if (isBelow)
                graphics2D.drawString(print, annotationPos, MID_LINE + BIG_TICK_HEIGHT + 10 + 2);
            else graphics2D.drawString(print, annotationPos, MID_LINE - BIG_TICK_HEIGHT - 2);
        });

    }

    @Override
    public void onDataSourceChanged(DataSettings deltaSettings) {
        SwingUtilities.invokeLater(this::repaint);
    }

    @Override
    public void onVisualSettingsChanged() {
        SwingUtilities.invokeLater(this::repaint);
    }

}
