package org.processmining.performancespectrumintegration.mypsm;

import com.fluxicon.slickerbox.factory.SlickerFactory;
import org.processmining.performancespectrumintegration.models.settings.CaseIdClassification;
import org.processmining.performancespectrumintegration.mypsm.update.PSMUpdateManager;

import javax.swing.*;
import java.awt.*;
import java.util.Map;
import java.util.SortedMap;

public class ClassificationLegendPanel extends JPanel {

    private final PSMUpdateManager psmUpdateManager;
    private JPanel content;


    public ClassificationLegendPanel(PSMUpdateManager psmUpdateManager) {
        this.psmUpdateManager = psmUpdateManager;

        content = new JPanel(new GridBagLayout());

        add(content);

        reset();
    }

    private void reset() {
        if (psmUpdateManager.getDataSource().getClassification() == null) return;

        GridBagConstraints c = new GridBagConstraints();
        c.ipadx = 20;
        c.gridy = 0;
        c.weighty = 1;
        c.weightx = 1;
        c.fill = GridBagConstraints.NONE;

        CaseIdClassification classification = psmUpdateManager.getDataSource().getClassification();
        SortedMap<Integer, Integer> classSizes = classification.getClassSizes();
        int sum = classSizes.values().stream().mapToInt(i -> i).sum();
        PSMColorMap psmColorMap = psmUpdateManager.getPsmColorMap();
        for (Map.Entry<Integer, String> entry : classification.getLegend().entrySet()) {
            Integer classId = entry.getKey();
            String description = entry.getValue();
            Color color = psmColorMap.getColor(classId);
            int size = classSizes.get(classId);
            String format = String.format("%d/%d cases [%.2f%%]", size, sum, 100 * size / (double) sum);
            JCheckBox checkBox = SlickerFactory.instance().createCheckBox("Class " + classId + " (" + format + ")", psmColorMap.isActive(classId));
            checkBox.addActionListener(e -> SwingUtilities.invokeLater(() -> {
                psmColorMap.setActive(classId, checkBox.isSelected());
                psmUpdateManager.colorMapChanged();
            }));

            c.gridx = 0;
            c.anchor = GridBagConstraints.WEST;
            content.add(checkBox, c);

            JButton colorButton = new JButton("");
            colorButton.addActionListener(e -> SwingUtilities.invokeLater(() -> {
                Color newColor = JColorChooser.showDialog(this, "Colorpicker for class " + classId, color);
                psmColorMap.setColor(classId, newColor);
                psmUpdateManager.colorMapChanged();
                colorButton.setBackground(newColor);
            }));
            colorButton.setBackground(color);

            c.gridx++;
            c.fill = GridBagConstraints.BOTH;
            c.anchor = GridBagConstraints.CENTER;
            content.add(colorButton, c);
            c.fill = GridBagConstraints.NONE;

            c.gridx++;
            c.anchor = GridBagConstraints.EAST;
            content.add(SlickerFactory.instance().createLabel(description), c);
            c.gridy++;
        }

        content.revalidate();
        content.repaint();
        revalidate();
        repaint();
    }

}
