package org.processmining.performancespectrumintegration.mypsm;

import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import org.processmining.performancespectrumintegration.models.settings.CaseIdClassification;

import java.awt.*;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import java.util.SortedMap;

public class PSMColorMap {

    private static final Color[] MY_COLORS = new Color[]{
            //new Color(0xf1faee),
            //new Color(0xead2ac),
            //new Color(0xe6b89c),
            new Color(0xfe938c),
            new Color(0x1d3557),
            new Color(0xa8dadc),
            new Color(0xe63946),
            new Color(0x4281a4),
            new Color(0x9cafb7),
    };

    private static final Color[] COLOR_BLIND = new Color[]{
            new Color(0xfc8d62),
            new Color(0x66c2a5),
            new Color(0x8da0cb),
    };

    private static final Color[] COLOR_BREWER = new Color[]{
            new Color(0xe41a1c),
            new Color(0x377eb8),
            new Color(0x4daf4a),
            new Color(0x984ea3),
            new Color(0xff7f00),
    };

    private SortedMap<Integer, Color> colorMap;
    private Map<Integer, Boolean> activeMap;
    private Set<Color> colorSet;

    public PSMColorMap() {
        colorMap = Maps.newTreeMap();
        activeMap = Maps.newHashMap();
        colorSet = Sets.newHashSet();
    }

    public static PSMColorMap createFrom(CaseIdClassification classification) {
        PSMColorMap result = new PSMColorMap();
        result.reset(classification);
        return result;
    }

    public void reset(CaseIdClassification classification) {
        colorMap.clear();
        activeMap.clear();
        colorSet.clear();

        if (classification.getNumberOfClasses() <= COLOR_BLIND.length) {
            int i = 0;
            for (Integer aClass : classification.getClasses()) {
                colorMap.put(aClass, COLOR_BLIND[i]);
                activeMap.put(aClass, true);
                colorSet.add(COLOR_BLIND[i]);
                i++;
            }
        } else if (classification.getNumberOfClasses() <= COLOR_BREWER.length) {
            int i = 0;
            for (Integer aClass : classification.getClasses()) {
                colorMap.put(aClass, COLOR_BREWER[i]);
                activeMap.put(aClass, true);
                colorSet.add(COLOR_BREWER[i]);
                i++;
            }
        } else {
            Random random = new Random();

            for (Integer aClass : classification.getClasses()) {
                final float hue = random.nextFloat();
                final float saturation = (random.nextInt(2000) + 1000) / 10000f;
                final float luminance = 0.9f;
                final Color color = Color.getHSBColor(hue, saturation, luminance);
                colorMap.put(aClass, color);
                activeMap.put(aClass, true);
                colorSet.add(color);
            }
        }
    }

    public void setActive(Integer classId, boolean active) {
        activeMap.put(classId, active);
    }

    public void setColor(Integer classId, Color color) {
        colorMap.put(classId, color);
    }

    public boolean isActive(Integer classId) {
        return activeMap.containsKey(classId) && activeMap.get(classId);
    }

    public Color getColor(Integer classId) {
        return colorMap.get(classId);
    }

}
