package org.processmining.performancespectrumintegration.mypsm;

import org.processmining.performancespectrumintegration.models.MyDataSource;
import org.processmining.performancespectrumintegration.mypsm.update.PSMUpdateManager;

import javax.swing.*;
import java.awt.*;

public class ViewScalePanel extends JPanel {

    public static final int BOX_HEIGHT = 12;
    private final PSMUpdateManager psmUpdateManager;

    public ViewScalePanel(PSMUpdateManager psmUpdateManager) {
        this.psmUpdateManager = psmUpdateManager;
        setPreferredSize(new Dimension(200, BOX_HEIGHT + 4));
    }

    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);

        Graphics2D graphics2D = (Graphics2D) g;

        int width = getWidth();
        int height = getHeight();

        graphics2D.setStroke(new BasicStroke(2));
        graphics2D.drawLine(0, height / 2, width, height / 2);
        graphics2D.drawLine(1, height / 2 - BOX_HEIGHT - 2, 1, height / 2 + BOX_HEIGHT + 2);
        graphics2D.drawLine(width - 1, height / 2 - BOX_HEIGHT - 2, width - 1, height / 2 + BOX_HEIGHT + 2);

        MyDataSource dataSource = psmUpdateManager.getDataSource();

        if (dataSource.isComplete()) {
            ViewWindowTools.WindowParams viewWindow = ViewWindowTools.getWindow(psmUpdateManager.getVisualSettings(), dataSource.getProcessStart(), dataSource.getProcessDuration(), width);

            long relWindowStart = viewWindow.getRelWindowStart();
            long windowSize = viewWindow.getWindowSize();

            double scale = width / (double) dataSource.getProcessDuration().getMillis();
            int boxStart = (int) (scale * relWindowStart);
            int boxWidth = Math.max((int) Math.round(scale * windowSize), 1);

            graphics2D.fillRect(boxStart, height / 2 - BOX_HEIGHT / 2, boxWidth, BOX_HEIGHT);

        }

    }


}
