package org.processmining.performancespectrumintegration.mypsm;

import org.joda.time.DateTime;
import org.joda.time.Duration;
import org.processmining.performancespectrumintegration.mypsm.update.PSMVisualSettings;

public class ViewWindowTools {

    public static WindowParams getWindow(PSMVisualSettings psmVisualSettings, DateTime processStart, Duration processDuration, int width) {
        double duration = processDuration.getMillis();
        double scale = psmVisualSettings.getXScale();
        double anchor = psmVisualSettings.getViewAnchor();

        double pxperms = scale * width / duration;

        double windowSize = duration / scale;
        //                = 160ms/px * 400px = 96000ms / 1.5 = 64000ms
        double windowAnchor = duration * anchor;
        //                  = 96000ms * 0.5 = 48000ms
        double windowStart;
        if (windowAnchor >= duration - windowSize / 2)
            windowStart = duration - windowSize;
        else if (windowAnchor <= windowSize / 2)
            windowStart = 0;
        else
            windowStart = windowAnchor - windowSize / 2;

        return new WindowParams(processStart, (long) windowStart, (long) windowSize, pxperms, processDuration);
    }

    public static class WindowParams {
        private final Duration processDuration;
        private long relWindowStart;
        private DateTime processStart;
        private DateTime windowStart;
        private DateTime windowEnd;
        private long windowSize;
        private double pixelPerMs;

        public WindowParams(DateTime processStart, long relWindowStart, long windowSize, double pixelPerMs, Duration processDuration) {
            this.relWindowStart = relWindowStart;
            this.processStart = processStart;
            this.processDuration = processDuration;
            this.windowStart = processStart.plus(relWindowStart);
            this.windowSize = windowSize;
            this.windowEnd = windowStart.plus(windowSize);
            this.pixelPerMs = pixelPerMs;
        }

        public long getRelWindowStart() {
            return relWindowStart;
        }

        public DateTime getWindowStart() {
            return windowStart;
        }

        public long getWindowSize() {
            return windowSize;
        }

        public double getPixelPerMs() {
            return pixelPerMs;
        }

        public DateTime getWindowEnd() {
            return windowEnd;
        }

        public DateTime getProcessStart() {
            return processStart;
        }

        public Duration getProcessDuration() {
            return processDuration;
        }
    }
}
