package org.processmining.performancespectrumintegration.mypsm.segmentpanel;

import com.fluxicon.slickerbox.factory.SlickerFactory;
import org.processmining.performancespectrumintegration.algorithms.segments.Segment;
import org.processmining.performancespectrumintegration.mypsm.update.PSMUpdateManager;
import org.processmining.performancespectrumintegration.mypsm.update.PlaceSegmentConnection;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Collection;

public class SegmentControlPanel extends UpdatableJPanel {

    private final SegmentPanel segmentPanel;
    private final PSMUpdateManager psmUpdateManager;
    private final JPanel hoverBar;

    public SegmentControlPanel(SegmentPanel segmentPanel, PSMUpdateManager psmUpdateManager) {
        this.segmentPanel = segmentPanel;
        this.psmUpdateManager = psmUpdateManager;

        updatePreferredHeight();
        setLayout(new GridBagLayout());

        hoverBar = SlickerFactory.instance().createRoundedPanel(5, Color.gray);
        hoverBar.setPreferredSize(new Dimension(20, 200));
        hoverBar.setSize(20, 200);
        hoverBar.addMouseListener(new MouseListener() {
            @Override
            public void mouseClicked(MouseEvent e) {

            }

            @Override
            public void mousePressed(MouseEvent e) {

            }

            @Override
            public void mouseReleased(MouseEvent e) {

            }

            @Override
            public void mouseEntered(MouseEvent e) {
                PlaceSegmentConnection conn = psmUpdateManager.getPlaceSegmentConnection();
                conn.markSegment(segmentPanel.getSegment());
                conn.applyHighlighting();
                psmUpdateManager.placeSegmentConnectionChanged();
            }

            @Override
            public void mouseExited(MouseEvent e) {
                PlaceSegmentConnection conn = psmUpdateManager.getPlaceSegmentConnection();
                conn.markSegment(null);
                conn.applyHighlighting();
                psmUpdateManager.placeSegmentConnectionChanged();
            }
        });

        JButton upButton = SlickerFactory.instance().createButton("up");
        upButton.addActionListener(e -> segmentPanel.moveUp());
        JButton downButton = SlickerFactory.instance().createButton("down");
        downButton.addActionListener(e -> segmentPanel.moveDown());
        JButton linesButton = SlickerFactory.instance().createButton("lines");
        linesButton.addActionListener(e -> segmentPanel.showLinesPanel());
        JButton barsButton = SlickerFactory.instance().createButton("agg");
        barsButton.addActionListener(e -> segmentPanel.showBarsPanel());
        JButton infoButton = SlickerFactory.instance().createButton("info");
        infoButton.addActionListener(e -> segmentPanel.showInfoPanel());

        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.BOTH;
        c.anchor = GridBagConstraints.WEST;
        c.gridy = 0;
        c.gridx = 0;
        c.weighty = 10;
        c.weightx = 0;
        c.gridheight = 5;
        add(hoverBar, c);

        c.gridheight = 1;
        c.weighty = 0;
        c.weightx = 1;
        c.gridx = 1;
        c.gridy = 0;
        c.anchor = GridBagConstraints.NORTH;
        c.fill = GridBagConstraints.NONE;
        add(upButton, c);
        c.gridy++;
        add(downButton, c);

        c.anchor = GridBagConstraints.SOUTH;
        c.weighty = 1;
        c.gridy++;
        add(linesButton, c);
        c.weighty = 0;
        c.gridy++;
        add(barsButton, c);
        c.gridy++;
        add(infoButton, c);

        /*
        JButton x = SlickerFactory.instance().createButton("x");
        x.addActionListener(e -> segmentPanel.removeItself());
        add(x, BorderLayout.SOUTH);
        */

        updateHighlighting();
    }

    public void updatePreferredHeight() {
        int newHeight = (int) (200 * psmUpdateManager.getVisualSettings().getYScale());
        setPreferredSize(new Dimension(70, newHeight));
        setSize(70, newHeight);
    }

    @Override
    public void updateHighlighting() {
        PlaceSegmentConnection conn = psmUpdateManager.getPlaceSegmentConnection();
        Collection<Segment> highlightedSegments = conn.getHighlightedSegments();

        if (highlightedSegments.contains(segmentPanel.getSegment()))
            hoverBar.setBackground(Color.red);
        else hoverBar.setBackground(Color.gray);

        repaint();
    }
}
