package org.processmining.performancespectrumintegration.mypsm.segmentpanel;

import com.fluxicon.slickerbox.factory.SlickerFactory;
import org.joda.time.Duration;
import org.processmining.performancespectrumintegration.algorithms.segments.Segment;
import org.processmining.performancespectrumintegration.models.settings.DataSettings;
import org.processmining.performancespectrumintegration.mypsm.update.PSMUpdateManager;
import org.processmining.performancespectrumintegration.util.ToStrings;

import javax.swing.*;
import java.awt.*;
import java.util.LongSummaryStatistics;
import java.util.concurrent.ExecutionException;

public class SegmentInfoPanel extends UpdatableJPanel {

    private final Segment segment;
    private final PSMUpdateManager psmUpdateManager;

    public SegmentInfoPanel(Segment segment, PSMUpdateManager psmUpdateManager) {
        this.segment = segment;
        this.psmUpdateManager = psmUpdateManager;

        setLayout(new GridBagLayout());

        reset();
    }

    private void reset() {
        removeAll();

        GridBagConstraints c = new GridBagConstraints();
        c.anchor = GridBagConstraints.NORTHWEST;
        c.fill = GridBagConstraints.NONE;

        JLabel label = SlickerFactory.instance().createLabel(segment.toString());
        add(label, c);

        JTextArea jTextArea = new JTextArea();
        jTextArea.setEditable(false);

        int filteredCases = psmUpdateManager.getDataSource().getFilteredCases();
        int totalCases = psmUpdateManager.getDataSource().getTotalCases();

        jTextArea.append("Counted Occurrences:\t");
        jTextArea.append(psmUpdateManager.getDataSource().getSegmentReliability(segment).toString() + " (due to moves on model in alignment)");
        jTextArea.append("\n");
        jTextArea.append("Counted Cases:\t");
        jTextArea.append(filteredCases + "/" + totalCases + " [" + String.format("%.2f%%", 100 * (double) filteredCases / totalCases) + "] (due to filtering)");
        jTextArea.append("\n");

        SwingWorker<LongSummaryStatistics, Void> sw = new SwingWorker<LongSummaryStatistics, Void>() {
            @Override
            protected LongSummaryStatistics doInBackground() throws Exception {
                return psmUpdateManager.getDataSource().getSegmentMeasurement(segment).getOccurrences().stream().mapToLong(so -> so.getDuration().getMillis()).summaryStatistics();
            }

            @Override
            protected void done() {
                try {
                    LongSummaryStatistics stats = get();
                    if (stats.getCount() > 0) {
                        Duration min = new Duration(stats.getMin());
                        Duration avg = new Duration((long) stats.getAverage());
                        Duration max = new Duration(stats.getMax());

                        jTextArea.append("Minimum Duration:\t" + ToStrings.print(min));
                        jTextArea.append("\n");
                        jTextArea.append("Average Duration:\t" + ToStrings.print(avg));
                        jTextArea.append("\n");
                        jTextArea.append("Maximum Duration:\t" + ToStrings.print(max));
                        jTextArea.append("\n");
                    }
                } catch (InterruptedException | ExecutionException e) {
                    e.printStackTrace();
                }
            }
        };

        sw.execute();

        c.fill = GridBagConstraints.BOTH;
        c.gridy = 1;
        c.weighty = 1;
        c.weightx = 1;
        c.anchor = GridBagConstraints.SOUTH;
        add(jTextArea, c);
    }


    @Override
    public void updateDataSettings(DataSettings deltaSettings) {
        if (deltaSettings.getFilterSettings() != null) {
            reset();
            revalidate();
            repaint();
        }
    }
}
