package org.processmining.performancespectrumintegration.mypsm.segmentpanel;

import org.processmining.performancespectrumintegration.algorithms.segments.Segment;
import org.processmining.performancespectrumintegration.models.settings.DataSettings;
import org.processmining.performancespectrumintegration.mypsm.SegmentsListPanel;
import org.processmining.performancespectrumintegration.mypsm.update.PSMUpdateManager;

import javax.swing.*;
import java.awt.*;

public class SegmentPanel extends UpdatableJPanel {

    private final Segment segment;
    private final PSMUpdateManager psmUpdateManager;
    private final SegmentsListPanel segmentsListPanel;
    private final SegmentControlPanel segmentControlPanel;
    private UpdatableJPanel rightPanel;

    public SegmentPanel(Segment segment, PSMUpdateManager psmUpdateManager, SegmentsListPanel segmentsListPanel) {
        this.segment = segment;
        this.psmUpdateManager = psmUpdateManager;
        this.segmentsListPanel = segmentsListPanel;

        setLayout(new GridBagLayout());

        segmentControlPanel = new SegmentControlPanel(this, psmUpdateManager);
        rightPanel = new SegmentLinesPanel(segment, psmUpdateManager);

        GridBagConstraints c = new GridBagConstraints();
        c.anchor = GridBagConstraints.EAST;
        c.fill = GridBagConstraints.BOTH;
        c.weightx = 0;
        c.weighty = 1;
        c.gridx = 0;
        add(segmentControlPanel, c);

        addRightPanel();

        updatePreferredHeight();
    }

    private void addRightPanel() {
        GridBagConstraints c = new GridBagConstraints();
        c.anchor = GridBagConstraints.WEST;
        c.fill = GridBagConstraints.BOTH;
        c.weightx = 1;
        c.weighty = 1;
        c.gridx = 1;
        add(rightPanel, c);
    }

    public void showLinesPanel() {
        SwingUtilities.invokeLater(() -> {
            setRightPanel(new SegmentLinesPanel(segment, psmUpdateManager));
        });
    }

    public void showBarsPanel() {
        SwingUtilities.invokeLater(() -> {
            setRightPanel(new SegmentBarsPanel(segment, psmUpdateManager));
        });
    }

    public void showInfoPanel() {
        SwingUtilities.invokeLater(() -> {
            setRightPanel(new SegmentInfoPanel(segment, psmUpdateManager));
        });
    }

    private void setRightPanel(UpdatableJPanel panel) {
        remove(1);
        rightPanel = panel;
        addRightPanel();
        revalidate();
        repaint();
    }

    @Override
    public void updateDataSettings(DataSettings deltaSettings) {
        if (rightPanel != null) rightPanel.updateDataSettings(deltaSettings);
    }

    private void updatePreferredHeight() {
        double yScale = psmUpdateManager.getVisualSettings().getYScale();
        int newHeight = (int) (200 * yScale);
        setPreferredSize(new Dimension(300, newHeight));
        segmentControlPanel.updatePreferredHeight();
    }

    @Override
    public void updateVisualSettings() {
        updatePreferredHeight();
        if (rightPanel instanceof SegmentDataPanel) {
            rightPanel.updateVisualSettings();
        }
    }

    @Override
    public void updateHighlighting() {
        segmentControlPanel.updateHighlighting();
    }

    public void moveUp() {
        segmentsListPanel.moveUp(this);
    }

    public void moveDown() {
        segmentsListPanel.moveDown(this);
    }

    public Segment getSegment() {
        return segment;
    }
}
