package org.processmining.performancespectrumintegration.mypsm.update;

import org.apache.commons.compress.utils.Lists;
import org.processmining.performancespectrumintegration.models.DataController;
import org.processmining.performancespectrumintegration.models.MyDataSource;
import org.processmining.performancespectrumintegration.models.settings.DataSettings;
import org.processmining.performancespectrumintegration.mypsm.PSMColorMap;

import java.util.List;

public class PSMUpdateManager {

    private final List<PSMUpdateListener> listeners;
    private final DataController<MyDataSource> dataController;

    private final PSMVisualSettings psmVisualSettings;
    private final PSMAggregationSettings psmAggregationSettings;
    private final PlaceSegmentConnection placeSegmentConnection;
    private final MarkedCaseConnection markedCaseConnection;
    private final SegmentOrderConnection segmentOrderConnection;
    private PSMColorMap psmColorMap;

    public PSMUpdateManager(DataController<MyDataSource> dataController) {
        this.dataController = dataController;
        listeners = Lists.newArrayList();
        psmVisualSettings = new PSMVisualSettings();
        psmColorMap = new PSMColorMap();
        psmAggregationSettings = new PSMAggregationSettings();
        placeSegmentConnection = new PlaceSegmentConnection();
        markedCaseConnection = new MarkedCaseConnection();
        segmentOrderConnection = new SegmentOrderConnection();
    }

    public void addListener(PSMUpdateListener listener) {
        listeners.add(listener);
    }

    public void removeListener(PSMUpdateListener listener) {
        listeners.remove(listener);
    }

    public PSMColorMap getPsmColorMap() {
        return psmColorMap;
    }

    public MyDataSource getDataSource() {
        return dataController.getDataSource();
    }

    public PSMVisualSettings getVisualSettings() {
        return psmVisualSettings;
    }

    public void visualSettingsChanged() {
        listeners.forEach(PSMUpdateListener::onVisualSettingsChanged);
    }

    public void placeSegmentConnectionChanged() {
        listeners.forEach(PSMUpdateListener::onPlaceSegmentConnectionChanged);
    }

    public void dataSourceGoingToChange(DataSettings deltaSettings) {

    }

    public void dataSourceChanged(DataSettings deltaSettings) {
        if (deltaSettings.getClassificationSettings() != null)
            psmColorMap = PSMColorMap.createFrom(dataController.getDataSource().getClassification());
        listeners.forEach(listener -> listener.onDataSourceChanged(deltaSettings));
    }

    public void markedCaseConnectionChanged() {
        listeners.forEach(PSMUpdateListener::onMarkedCaseConnectionChanged);
    }

    public void colorMapChanged() {
        listeners.forEach(PSMUpdateListener::onColorMapChanged);
    }

    public PSMAggregationSettings getPsmAggregationSettings() {
        return psmAggregationSettings;
    }

    public PlaceSegmentConnection getPlaceSegmentConnection() {
        return placeSegmentConnection;
    }

    public MarkedCaseConnection getMarkedCaseConnection() {
        return markedCaseConnection;
    }

    public SegmentOrderConnection getSegmentOrderConnection() {
        return segmentOrderConnection;
    }

}
