package org.processmining.performancespectrumintegration.mypsm.update;

import com.google.common.collect.Maps;
import com.google.common.collect.Multimap;
import com.google.common.collect.MultimapBuilder;
import com.google.common.collect.Sets;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.performancespectrumintegration.algorithms.segments.Segment;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;

public class PlaceSegmentConnection {

    private Multimap<Place, Segment> placeToSegments;
    private Map<Segment, Place> segmentToPlace;

    private Place markedPlace;
    private Segment markedSegment;

    private Collection<Segment> highlightedSegments;
    private Place highlightedPlace;

    public PlaceSegmentConnection() {
        placeToSegments = MultimapBuilder.hashKeys().arrayListValues().build();
        segmentToPlace = Maps.newHashMap();
        highlightedSegments = new ArrayList<>();
    }

    public void markSegment(Segment segment) {
        markedSegment = segment;
        markedPlace = null;
    }

    public void markPlace(Place place) {
        markedPlace = place;
        markedSegment = null;
    }

    public void applyHighlighting() {
        if (markedSegment != null && segmentToPlace.containsKey(markedSegment)) {
            highlightedPlace = segmentToPlace.get(markedSegment);
            highlightedSegments = placeToSegments.get(highlightedPlace);
        } else if (markedPlace != null && placeToSegments.containsKey(markedPlace)) {
            highlightedPlace = markedPlace;
            highlightedSegments = placeToSegments.get(markedPlace);
        } else if (markedPlace != null) {
            highlightedPlace = markedPlace;
            highlightedSegments = Sets.newHashSet();
        } else {
            highlightedPlace = null;
            highlightedSegments = Sets.newHashSet();
        }
    }

    public void add(Place p, Segment s) {
        placeToSegments.put(p, s);
        segmentToPlace.put(s, p);
    }

    public void reset() {
        placeToSegments.clear();
        segmentToPlace.clear();
        highlightedPlace = null;
        highlightedSegments.clear();
    }

    public Place getHighlightedPlace() {
        return highlightedPlace;
    }

    public Collection<Segment> getHighlightedSegments() {
        return highlightedSegments;
    }
}
