package org.processmining.performancespectrumintegration.parameters;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.processmining.models.graphbased.LocalNodeID;

import java.io.*;

public class ImportExportPSMParameters {

    public static void write(File file, PSMParameters psmParameters) throws IOException {
        FileOutputStream fileOutputStream = new FileOutputStream(file);
        try (ObjectOutputStream oos = new ObjectOutputStream(fileOutputStream)) {
            oos.writeObject(psmParameters);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void writeJson(File file, PSMParameters psmParameters) throws IOException {
        ObjectMapper mapper = new ObjectMapper();
        configureMapper(mapper);
        mapper.writeValue(file, psmParameters);
    }

    public static PSMParameters read(File file) throws IOException {
        FileInputStream fileInputStream = new FileInputStream(file);
        try (ObjectInputStream ois = new ObjectInputStream(fileInputStream)) {
            return (PSMParameters) ois.readObject();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static PSMParameters read(InputStream inputStream) throws IOException {
        try (ObjectInputStream ois = new ObjectInputStream(inputStream)) {
            return (PSMParameters) ois.readObject();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static PSMParameters readJson(File file) throws IOException {
        ObjectMapper mapper = new ObjectMapper();
        configureMapper(mapper);
        return mapper.readValue(file, PSMParameters.class);
    }

    private static void configureMapper(ObjectMapper mapper) {
        mapper.enableDefaultTyping();
        // mapper.registerModule(new JodaModule());
        mapper.addMixIn(LocalNodeID.class, LocalNodeIDMixIn.class);
    }

    public static String toJson(PSMParameters psmParameters) {
        ObjectMapper mapper = new ObjectMapper();
        configureMapper(mapper);
        try {
            return mapper.writerWithDefaultPrettyPrinter().writeValueAsString(psmParameters);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return "";
    }

    public static PSMParameters fromJson(String json) {
        ObjectMapper mapper = new ObjectMapper();
        configureMapper(mapper);
        try {
            return mapper.readValue(json, PSMParameters.class);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

}
