package org.processmining.performancespectrumintegration.parameters;

import com.fasterxml.jackson.annotation.JsonProperty;
import org.processmining.models.graphbased.LocalNodeID;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.performancespectrumintegration.models.settings.DataSettings;
import org.processmining.performancespectrumintegration.parameters.serialization.PrimitiveDataSettings;
import org.processmining.performancespectrumintegration.parameters.serialization.PrimitivePlaceSelectionMap;
import org.processmining.performancespectrumintegration.visualization.PlaceSelectionMap;
import org.processmining.performancespectrumintegration.visualization.controller.SegmentMeasurementController;

import java.io.Serializable;
import java.util.Optional;

public class PSMParameters implements Serializable {

    @JsonProperty
    private boolean instantMode;
    @JsonProperty
    private PrimitiveDataSettings primitiveDataSettings;
    @JsonProperty
    private PrimitivePlaceSelectionMap primitivePlaceSelectionMap;

    public PSMParameters() {
    }

    public PSMParameters(DataSettings dataSettings, PlaceSelectionMap placeSelectionMap, boolean instantMode) {
        primitiveDataSettings = new PrimitiveDataSettings(dataSettings);
        primitivePlaceSelectionMap = new PrimitivePlaceSelectionMap(placeSelectionMap);
        this.instantMode = instantMode;
    }

    public static Place findPlace(LocalNodeID localNodeID, Petrinet net) {
        Optional<Place> first = net.getPlaces().stream().filter(p -> p.getLocalID().toString().equals(localNodeID.toString())).findFirst();
        return first.orElse(null);
    }

    public static Transition findTransition(LocalNodeID localNodeID, Petrinet net) {
        Optional<Transition> first = net.getTransitions().stream().filter(t -> t.getLocalID().toString().equals(localNodeID.toString())).findFirst();
        return first.orElse(null);
    }

    @Override
    public String toString() {
        return "PSMParameters{" +
                "primitiveDataSettings=" + primitiveDataSettings +
                ", primitivePlaceSelectionMap=" + primitivePlaceSelectionMap +
                '}';
    }

    public void recreate(DataSettings dataSettings, PlaceSelectionMap placeSelectionMap, SegmentMeasurementController measurementController, Petrinet original) {
        primitiveDataSettings.recreate(dataSettings, original);
        primitivePlaceSelectionMap.recreate(placeSelectionMap, measurementController, original);
    }

    public boolean isInstantMode() {
        return instantMode;
    }
}
