package org.processmining.performancespectrumintegration.parameters.serialization;

import com.fasterxml.jackson.annotation.JsonProperty;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.performancespectrumintegration.models.settings.*;

import java.io.Serializable;

public class PrimitiveDataSettings implements Serializable {

    @JsonProperty
    private PrimitiveSegmentOrder primitiveSegmentOrder;
    @JsonProperty
    private FilterSettings filterSettings;
    @JsonProperty
    private ClassificationSettings classificationSettings;
    @JsonProperty
    private AggregationIntervalSettings aggregationIntervalSettings;

    public PrimitiveDataSettings() {
    }

    public PrimitiveDataSettings(DataSettings dataSettings) {
        filterSettings = dataSettings.getFilterSettings();
        classificationSettings = dataSettings.getClassificationSettings();
        aggregationIntervalSettings = dataSettings.getAggregationIntervalSettings();
        SegmentOrderSettings segmentOrderSettings = dataSettings.getSegmentOrderSettings();
        if (segmentOrderSettings.usePredefined())
            primitiveSegmentOrder = new PrimitiveSegmentOrder(segmentOrderSettings.getPredefinedOrder());
    }

    public void recreate(DataSettings dataSettings, Petrinet net) {
        dataSettings.setFilterSettings(filterSettings);
        dataSettings.setClassificationSettings(classificationSettings);
        dataSettings.setAggregationIntervalSettings(aggregationIntervalSettings);
        if (primitiveSegmentOrder != null)
            dataSettings.setSegmentOrderSettings(new SegmentOrderSettings(primitiveSegmentOrder.recreate(net)));
        else dataSettings.setSegmentOrderSettings(SegmentOrderSettings.HEURISTIC);
    }

    @Override
    public String toString() {
        return "PrimitiveDataSettings{" +
                "filterSettings=" + filterSettings +
                ", classificationSettings=" + classificationSettings +
                ", aggregationIntervalSettings=" + aggregationIntervalSettings +
                ", primitiveSegmentOrder=" + primitiveSegmentOrder +
                '}';
    }
}
