package org.processmining.performancespectrumintegration.parameters.serialization;

import com.fasterxml.jackson.annotation.JsonProperty;
import org.processmining.models.graphbased.LocalNodeID;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.performancespectrumintegration.algorithms.segments.MultiSegment;
import org.processmining.performancespectrumintegration.algorithms.segments.Segment;
import org.processmining.performancespectrumintegration.algorithms.segments.SingleSegment;
import org.processmining.performancespectrumintegration.parameters.PSMParameters;
import org.processmining.performancespectrumintegration.util.Tuple2;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

public class PrimitiveSegmentOrder implements Serializable {

    @JsonProperty
    private List<Tuple2<Set<LocalNodeID>, Set<LocalNodeID>>> list;

    public PrimitiveSegmentOrder() {
    }

    public PrimitiveSegmentOrder(List<Segment> segmentOrder) {
        list = new LinkedList<>();
        segmentOrder.forEach(s -> list.add(convertSegmentToPrimitive(s)));
    }

    public static Tuple2<Set<LocalNodeID>, Set<LocalNodeID>> convertSegmentToPrimitive(Segment s) {
        return new Tuple2<>(s.getFrom().stream().map(Transition::getLocalID).collect(Collectors.toSet()), s.getTo().stream().map(Transition::getLocalID).collect(Collectors.toSet()));
    }

    @Override
    public String toString() {
        return "PrimitiveSegmentOrder{" +
                "list=" + list +
                '}';
    }

    public List<Segment> recreate(Petrinet net) {
        List<Segment> result = new LinkedList<>();
        for (Tuple2<Set<LocalNodeID>, Set<LocalNodeID>> tuple2 : list) {
            Set<Transition> from = tuple2._1.stream().map(id -> PSMParameters.findTransition(id, net)).filter(Objects::nonNull).collect(Collectors.toSet());
            Set<Transition> to = tuple2._2.stream().map(id -> PSMParameters.findTransition(id, net)).filter(Objects::nonNull).collect(Collectors.toSet());
            if (from.size() == 1 && to.size() == 1)
                result.add(new SingleSegment(from.stream().findFirst().get(), to.stream().findFirst().get()));
            else if (from.size() > 1 || to.size() > 1)
                result.add(new MultiSegment(from, to));
        }
        return result;
    }

}
