package org.processmining.performancespectrumintegration.util;

import org.joda.time.Duration;
import org.joda.time.Instant;
import org.joda.time.Period;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.joda.time.format.PeriodFormatter;
import org.joda.time.format.PeriodFormatterBuilder;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;

import java.util.stream.Collectors;


public class ToStrings {

    public static final PeriodFormatter rough = new PeriodFormatterBuilder()
            .appendYears().appendSuffix("y")
            .appendMonths().appendSuffix("M")
            .appendWeeks().appendSuffix("w")
            .appendDays().appendSuffix("d")
            .printZeroNever().toFormatter();

    public static final PeriodFormatter medium = new PeriodFormatterBuilder()
            .appendYears().appendSuffix("y")
            .appendMonths().appendSuffix("M")
            .appendWeeks().appendSuffix("w")
            .appendDays().appendSuffix("d")
            .appendHours().appendSuffix("h")
            .printZeroNever().toFormatter();

    public static final PeriodFormatter fine = new PeriodFormatterBuilder()
            .appendYears().appendSuffix("y")
            .appendMonths().appendSuffix("M")
            .appendWeeks().appendSuffix("w")
            .appendDays().appendSuffix("d")
            .appendHours().appendSuffix("h")
            .appendMinutes().appendSuffix("m")
            .appendSeconds().appendSuffix("s")
            .printZeroNever().toFormatter();

    public static final PeriodFormatter duration = new PeriodFormatterBuilder()
            .appendWeeks().appendSuffix("w")
            .appendDays().appendSuffix("d")
            .appendHours().appendSuffix("h")
            .appendMinutes().appendSuffix("m")
            .appendSeconds().appendSuffix("s")
            .printZeroNever().toFormatter();

    public static String print(Duration d) {
        return print(d.toPeriod().normalizedStandard(), getFittingPeriodFormatter(d));
    }

    public static String print(Period p) {
        return print(p, getFittingPeriodFormatter(p.toDurationFrom(new Instant(0))));
    }

    public static String print(Period p, PeriodFormatter formatter) {
        return formatter.print(p);
    }

    public static PeriodFormatter getFittingPeriodFormatter(Duration d) {
        if (d.isShorterThan(Duration.standardDays(2))) return fine;
        else if (d.isShorterThan(Duration.standardDays(7))) return medium;
        else return rough;
    }

    public static DateTimeFormatter getFittingDateTimeFormatter(Period p) {
        if (p.toDurationFrom(new Instant(0)).isLongerThan(Duration.standardDays(2)))
            return DateTimeFormat.shortDate();
        else
            return DateTimeFormat.shortDateTime();
    }

    public static String print(SyncReplayResult replayResult) {
        StringBuilder sb = new StringBuilder();
        sb.append("<");
        String collect = replayResult.getNodeInstance().stream().map(Object::toString).collect(Collectors.joining(","));
        sb.append(collect);
        sb.append(">");
        return sb.toString();
    }


}
