package org.processmining.performancespectrumintegration.util;

import org.deckfour.uitopia.ui.util.ImageLoader;
import org.joda.time.DateTime;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinet.impl.PetrinetFactory;
import org.processmining.performancespectrumintegration.models.settings.AggregationIntervalSettings;
import org.processmining.performancespectrumintegration.models.settings.IntervalPresetType;

import javax.swing.*;
import java.awt.image.BufferedImage;
import java.util.Map;

import static org.processmining.performancespectrumintegration.models.settings.IntervalPresetType.*;


public class Utils {

    public static boolean allNonNull(Object... obs) {
        if (obs == null) {
            return false;
        } else {
            for (Object ob : obs) {
                if (ob == null) {
                    return false;
                }
            }
            return true;
        }
    }

    public static DateTime normalizeToPeriod(DateTime dt, AggregationIntervalSettings aggIntSettings) {
        DateTime result = dt;

        IntervalPresetType presetType = aggIntSettings.getPresetType();

        if (presetType == Year) {
            result = dt.dayOfYear().withMinimumValue().withTimeAtStartOfDay();
        } else if (presetType == Quarter) {
            switch (dt.getMonthOfYear()) {
                case 1:
                case 2:
                case 3:
                    result = dt.withMonthOfYear(1).dayOfMonth().withMinimumValue().withTimeAtStartOfDay();
                    break;
                case 4:
                case 5:
                case 6:
                    result = dt.withMonthOfYear(4).dayOfMonth().withMinimumValue().withTimeAtStartOfDay();
                    break;
                case 7:
                case 8:
                case 9:
                    result = dt.withMonthOfYear(7).dayOfMonth().withMinimumValue().withTimeAtStartOfDay();
                    break;
                case 10:
                case 11:
                case 12:
                    result = dt.withMonthOfYear(10).dayOfMonth().withMinimumValue().withTimeAtStartOfDay();
                    break;
            }
        } else if (presetType == Month)
            result = dt.dayOfMonth().withMinimumValue().withTimeAtStartOfDay();
        else if (presetType == Week)
            result = dt.dayOfWeek().withMinimumValue().withTimeAtStartOfDay();
        else if (presetType == Day)
            result = dt.withTimeAtStartOfDay();
        else if (presetType == Hour)
            result = dt.minuteOfHour().withMinimumValue().secondOfMinute().withMinimumValue().millisOfSecond().withMinimumValue();
        else if (presetType == Minute)
            result = dt.secondOfMinute().withMinimumValue().millisOfSecond().withMinimumValue();

        return result;
    }


    public static ImageIcon loadImageIcon(String name) {
        BufferedImage load = ImageLoader.load(name);
        ImageIcon imageIcon = new ImageIcon();
        if (load != null) {
            imageIcon = new ImageIcon(load);
        } else { // TODO DEBUG ONLY
            imageIcon = new ImageIcon("lib/images/" + name);
        }
        return imageIcon;
    }


    // from AcceptingPetriNet package maps are original -> cloned
    public static Petrinet cloneNet(Petrinet net, Map<Place, Place> placeMap, Map<Transition, Transition> transitionMap, Map<Arc, Arc> arcMap) {
        Petrinet clonedNet = PetrinetFactory.newPetrinet(net.getLabel());
        for (Place place : net.getPlaces()) {
            Place clonedPlace = clonedNet.addPlace(place.getLabel());
            placeMap.put(place, clonedPlace);
            for (String key : place.getAttributeMap().keySet()) {
                clonedPlace.getAttributeMap().put(key, place.getAttributeMap().get(key));
            }
        }
        for (Transition transition : net.getTransitions()) {
            Transition clonedTransition = clonedNet.addTransition(transition.getLabel());
            clonedTransition.setInvisible(transition.isInvisible());
            transitionMap.put(transition, clonedTransition);
            for (String key : transition.getAttributeMap().keySet()) {
                clonedTransition.getAttributeMap().put(key, transition.getAttributeMap().get(key));
            }
        }
        for (PetrinetEdge<?, ?> edge : net.getEdges()) {
            if (edge instanceof Arc) {
                Arc arc = (Arc) edge;
                if (placeMap.containsKey(arc.getSource())) {
                    arcMap.put(arc, clonedNet
                            .addArc(placeMap.get(arc.getSource()), transitionMap.get(arc.getTarget()), arc.getWeight()));
                } else {
                    arcMap.put(arc, clonedNet
                            .addArc(transitionMap.get(arc.getSource()), placeMap.get(arc.getTarget()), arc.getWeight()));
                }
                for (String key : arc.getAttributeMap().keySet()) {
                    arcMap.get(arc).getAttributeMap().put(key, arc.getAttributeMap().get(key));
                }
            }
        }
        return clonedNet;
    }

}
