package org.processmining.performancespectrumintegration.visualization;

import com.google.common.collect.BiMap;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

import java.util.Set;

public class PetrinetCopyConnection {

    private final BiMap<Transition, Transition> transitionConnection; // original -> copy
    private final BiMap<Place, Place> placeConnection; // original -> copy
    private final BiMap<Arc, Arc> arcConnection; // original -> copy

    public PetrinetCopyConnection(BiMap<Transition, Transition> transitionConnection, BiMap<Place, Place> placeConnection, BiMap<Arc, Arc> arcConnection) {
        this.transitionConnection = transitionConnection;
        this.placeConnection = placeConnection;
        this.arcConnection = arcConnection;
    }

    public boolean hasOriginalTransition(Transition t) {
        return transitionConnection.inverse().containsKey(t);
    }

    public boolean hasOriginalPlace(Place p) {
        return placeConnection.inverse().containsKey(p);
    }

    public boolean hasOriginalArc(Arc a) {
        return arcConnection.inverse().containsKey(a);
    }

    public Set<Transition> getOriginalTransitions() {
        return transitionConnection.keySet();
    }

    public Set<Transition> getCopyTransitions() {
        return transitionConnection.values();
    }

    public Set<Place> getOriginalPlaces() {
        return placeConnection.keySet();
    }

    public Set<Place> getCopyPlaces() {
        return placeConnection.values();
    }

    public Set<Arc> getOriginalArcs() {
        return arcConnection.keySet();
    }

    public Set<Arc> getCopyArcs() {
        return arcConnection.values();
    }

    public Transition getCopyTransition(Transition t) {
        return transitionConnection.get(t);
    }

    public Transition getOriginalTransition(Transition t) {
        return transitionConnection.inverse().get(t);
    }

    public Place getCopyPlace(Place p) {
        return placeConnection.get(p);
    }

    public Place getOriginalPlace(Place p) {
        return placeConnection.inverse().get(p);
    }

    public Arc getCopyArc(Arc a) {
        return arcConnection.get(a);
    }

    public Arc getOriginalArc(Arc a) {
        return arcConnection.inverse().get(a);
    }

    public void putCopyTransition(Transition original, Transition copy) {
        transitionConnection.put(original, copy);
    }

    public void putCopyPlace(Place original, Place copy) {
        placeConnection.put(original, copy);
    }

    public void putCopyArc(Arc original, Arc copy) {
        arcConnection.put(original, copy);
    }

}
