package org.processmining.performancespectrumintegration.visualization.controller;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.labels.CategoryToolTipGenerator;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.StackedBarRenderer;
import org.jfree.chart.renderer.category.StandardBarPainter;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.statistics.HistogramDataset;
import org.joda.time.DateTime;
import org.joda.time.Duration;
import org.processmining.performancespectrumintegration.models.LogAlignmentInterface;
import org.processmining.performancespectrumintegration.models.settings.CaseIdFilter;
import org.processmining.performancespectrumintegration.models.settings.FilterSettings;
import org.processmining.performancespectrumintegration.util.ToStrings;
import org.processmining.performancespectrumintegration.util.Tuple2;

import java.text.FieldPosition;
import java.text.NumberFormat;
import java.text.ParsePosition;
import java.util.List;

public class ExtraComputationsController {

    public static final int VARIANT_BARCHART_WIDTH = 350;
    public static final int VARIANT_BARCHART_HEIGHT = 200;
    public static final int DURATION_HISTOGRAM_WIDTH = 400;
    public static final int DURATION_HISTOGRAM_HEIGHT = 250;
    private final LogAlignmentInterface logAlignmentInterface;

    public ExtraComputationsController(LogAlignmentInterface logAlignmentInterface) {
        this.logAlignmentInterface = logAlignmentInterface;
    }

    public ChartPanel getVariantBarChart() {
        return getVariantBarChart(null);
    }

    public ChartPanel getVariantBarChart(CaseIdFilter caseIdFilter) {
        Tuple2<CategoryDataset, List<String>> tuple = logAlignmentInterface.getVariantDistribution(caseIdFilter == null ? i -> true : caseIdFilter);
        CategoryDataset variantDistribution = tuple._1;
        List<String> variantLabels = tuple._2;
        JFreeChart stackedBarChart = ChartFactory.createStackedBarChart((caseIdFilter != null ? "Filtered " : "") + "Variant Distribution", "", "% of cases in variant", variantDistribution, PlotOrientation.HORIZONTAL, false, true, false);
        NumberAxis rangeAxis = (NumberAxis) stackedBarChart.getCategoryPlot().getRangeAxis();
        rangeAxis.setNumberFormatOverride(NumberFormat.getPercentInstance());
        StackedBarRenderer barrenderer = (StackedBarRenderer) stackedBarChart.getCategoryPlot().getRenderer();
        barrenderer.setBarPainter(new StandardBarPainter());
        barrenderer.setRenderAsPercentages(true);
        barrenderer.setBaseToolTipGenerator(new CategoryToolTipGenerator() {
            @Override
            public String generateToolTip(CategoryDataset dataset, int row, int column) {
                return variantLabels.get(row);
            }
        });

        ChartPanel chartPanel = new ChartPanel(stackedBarChart, VARIANT_BARCHART_WIDTH, VARIANT_BARCHART_HEIGHT, VARIANT_BARCHART_WIDTH, VARIANT_BARCHART_HEIGHT, VARIANT_BARCHART_WIDTH, VARIANT_BARCHART_HEIGHT, true, false, false, false, false, true);
        return chartPanel;
    }

    public ChartPanel getDurationHistogram() {
        return getDurationHistogram(null);
    }

    public ChartPanel getDurationHistogram(CaseIdFilter caseIdFilter) {
        HistogramDataset durationHistogramDataset = logAlignmentInterface.getDurationHistogramDataset(caseIdFilter == null ? i -> true : caseIdFilter);
        JFreeChart histogram = ChartFactory.createHistogram((caseIdFilter != null ? "Filtered " : "") + "Case Duration Distribution", "", "duration frequency", durationHistogramDataset, PlotOrientation.VERTICAL, false, false, false);
        NumberAxis rangeAxis = (NumberAxis) histogram.getXYPlot().getDomainAxis();
        rangeAxis.setNumberFormatOverride(new NumberFormat() {
            @Override
            public StringBuffer format(double number, StringBuffer toAppendTo, FieldPosition pos) {
                return toAppendTo.append(ToStrings.print(new Duration((long) number)));
            }

            @Override
            public StringBuffer format(long number, StringBuffer toAppendTo, FieldPosition pos) {
                return toAppendTo.append(ToStrings.print(new Duration(number)));
            }

            @Override
            public Number parse(String source, ParsePosition parsePosition) {
                return null;
            }
        });
        ((NumberAxis) histogram.getXYPlot().getRangeAxis()).setNumberFormatOverride(NumberFormat.getPercentInstance());

        ChartPanel chartPanel = new ChartPanel(histogram, DURATION_HISTOGRAM_WIDTH, DURATION_HISTOGRAM_HEIGHT, DURATION_HISTOGRAM_WIDTH, DURATION_HISTOGRAM_HEIGHT, DURATION_HISTOGRAM_WIDTH, DURATION_HISTOGRAM_HEIGHT, true, false, false, false, false, true);
        //chartPanel.setPreferredSize(new Dimension(250, 250));
        return chartPanel;
    }

    public Tuple2<DateTime, DateTime> getProcessDuration(CaseIdFilter caseIdFilter) {
        Tuple2<DateTime, DateTime> call = null;
        try {
            call = logAlignmentInterface.buildProcessDuration(caseIdFilter).call();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return call;
    }

    public CaseIdFilter getFilter(FilterSettings filterSettings) {
        CaseIdFilter call = null;
        try {
            call = logAlignmentInterface.buildFilter(filterSettings).call();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return call;
    }
}
