package org.processmining.performancespectrumintegration.visualization.controller;

import org.processmining.models.graphbased.ViewSpecificAttributeMap;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.performancespectrumintegration.visualization.PlaceSelectionMap;
import org.processmining.performancespectrumintegration.visualization.PlaceSelectionState;

import java.util.Collection;

public class PlaceSelectionController {

    private final GraphController graphController;
    private final PlaceSelectionMap placeSelectionMap;

    public PlaceSelectionController(GraphController graphController, PlaceSelectionMap placeSelectionMap) {
        this.graphController = graphController;
        this.placeSelectionMap = placeSelectionMap;
    }

    public void setState(Place place, PlaceSelectionState state) {
        placeSelectionMap.setState(place, state, graphController.getAttributeMap());
        graphController.updatedAttributes(place);
    }

    public PlaceSelectionState getState(Place place) {
        return placeSelectionMap.getState(place);
    }

    public void updateAttributes(ViewSpecificAttributeMap map) {
        placeSelectionMap.updateAttributes(map);
    }

    public void toggleState(Place place) {
        placeSelectionMap.toggleState(place, graphController.getAttributeMap());
        graphController.updatedAttributes(place);
    }

    public void toggleStates(Collection<Place> places) {
        for (Place p : places) {
            placeSelectionMap.toggleState(p, graphController.getAttributeMap());
        }
        graphController.updatedAttributes(places);
    }

    public void resetAll() {
        graphController.updatedAttributes(placeSelectionMap.resetAll(graphController.getAttributeMap()));
    }

}
