package org.processmining.performancespectrumintegration.visualization.inputpanels;

import com.fluxicon.slickerbox.factory.SlickerFactory;
import org.jfree.chart.ChartPanel;
import org.joda.time.Duration;
import org.joda.time.Instant;
import org.joda.time.Period;
import org.processmining.performancespectrumintegration.models.settings.*;
import org.processmining.performancespectrumintegration.util.ToStrings;
import org.processmining.performancespectrumintegration.visualization.SettingsPanel;
import org.processmining.performancespectrumintegration.visualization.controller.ExtraComputationsController;

import javax.swing.*;
import javax.swing.text.JTextComponent;
import java.awt.*;
import java.util.Arrays;
import java.util.Set;
import java.util.concurrent.ExecutionException;
import java.util.function.BiFunction;
import java.util.function.Function;
import java.util.function.Supplier;
import java.util.stream.Collectors;

import static org.processmining.performancespectrumintegration.visualization.controller.ExtraComputationsController.*;

/**
 * I apologize for this abomination
 *
 * @param <T>
 */
public class DialogPanel<T> extends JPanel {

    private Supplier<T> resultExtractor;
    private GridBagConstraints c;
    private String title;

    public DialogPanel() {
        super(new GridBagLayout());
        c = new GridBagConstraints();
        c.weightx = 1;
        c.weighty = 1;
        c.gridx = 0;
        c.gridy = 0;
        c.fill = GridBagConstraints.BOTH;
        c.anchor = GridBagConstraints.NORTHWEST;
    }

    public static DialogPanel<PercentageFilterSettings> createDurationPercentileFilter(FilterSettings initialSettings, ExtraComputationsController extraComputationsController) {
        return createPercentageFilterWithGraph(initialSettings, FilterType.CaseDurationPercentile, extraComputationsController::getDurationHistogram, DURATION_HISTOGRAM_WIDTH, DURATION_HISTOGRAM_HEIGHT, "Duration Filter Settings", "[%] shortest/longest cases", "shortest", "longest");
    }

    public static DialogPanel<PercentageFilterSettings> createVariantPercentileFilter(FilterSettings initialSettings, ExtraComputationsController extraComputationsController) {
        return createPercentageFilterWithGraph(initialSettings, FilterType.VariantPercentile, extraComputationsController::getVariantBarChart, VARIANT_BARCHART_WIDTH, VARIANT_BARCHART_HEIGHT, "Variant Filter Settings", "[%] most/least frequent cases", "most frequent", "least frequent");
    }

    public static DialogPanel<PercentageFilterSettings> createPercentageFilterWithGraph(FilterSettings initialSettings, FilterType type, Supplier<ChartPanel> chartPanelSupplier, int graphWidth, int graphHeight, String title, String labelString, String inc, String exc) {
        JPanel graphPanel = new JPanel(new BorderLayout());
        graphPanel.add(new JLabel(SettingsPanel.loadingIcon), BorderLayout.CENTER);
        graphPanel.setPreferredSize(new Dimension(graphWidth, graphHeight));

        DialogPanel<PercentageFilterSettings> panel = createIncExcString(labelString, inc, exc, (String s, Boolean inverted) -> {
            try {
                double v = Double.parseDouble(s) / 100;
                return new PercentageFilterSettings(type, v, inverted);
            } catch (NumberFormatException e) {
            }
            return null;
        }, ((jToggleButton) -> {
            JTextField jTextField = new JTextField("50");
            if (initialSettings instanceof PercentageFilterSettings && initialSettings.getType() == type) {
                PercentageFilterSettings settings = (PercentageFilterSettings) initialSettings;
                jTextField.setText(String.valueOf(settings.getPercentage() * 100));
                jToggleButton.setSelected(settings.isInverted());
            } else {
                jToggleButton.setSelected(false);
            }
            return jTextField;
        }), graphPanel);

        panel.setTitle(title);

        SwingWorker<ChartPanel, Void> sw = new SwingWorker<ChartPanel, Void>() {
            @Override
            protected ChartPanel doInBackground() throws Exception {
                return chartPanelSupplier.get();
            }

            @Override
            protected void done() {
                try {
                    ChartPanel chartPanel = get();
                    graphPanel.removeAll();
                    graphPanel.add(chartPanel, BorderLayout.CENTER);
                    graphPanel.revalidate();
                    graphPanel.repaint();
                    panel.revalidate();
                    panel.repaint();
                } catch (InterruptedException | ExecutionException e) {
                    e.printStackTrace();
                }
            }
        };

        sw.execute();

        return panel;
    }

    public static DialogPanel<DurationThresholdFilterSettings> createDurationThresholdFilter(FilterSettings initialSettings, ExtraComputationsController extraComputationsController) {
        return createThresholdFilterWithGraph(initialSettings, FilterType.CaseDurationThreshold, extraComputationsController::getDurationHistogram, DURATION_HISTOGRAM_WIDTH, DURATION_HISTOGRAM_HEIGHT, "Duration Filter Settings", "Threshold Duration [ex. 1w2d3h4m5s]", "include shorter", "include longer");
    }

    public static DialogPanel<DurationThresholdFilterSettings> createThresholdFilterWithGraph(FilterSettings initialSettings, FilterType type, Supplier<ChartPanel> chartPanelSupplier, int graphWidth, int graphHeight, String title, String labelString, String inc, String exc) {
        JPanel graphPanel = new JPanel(new BorderLayout());
        graphPanel.add(new JLabel(SettingsPanel.loadingIcon), BorderLayout.CENTER);
        graphPanel.setPreferredSize(new Dimension(graphWidth, graphHeight));

        DialogPanel<DurationThresholdFilterSettings> panel = createIncExcString(labelString, inc, exc, (String s, Boolean inverted) -> {
            try {
                Period period = Period.parse(s, ToStrings.duration);
                Duration duration = period.normalizedStandard().toDurationFrom(new Instant(0));
                return new DurationThresholdFilterSettings(duration, inverted);
            } catch (IllegalArgumentException ex) {
            }
            return null;
        }, ((jToggleButton) -> {
            JTextField jTextField = new JTextField("");
            if (initialSettings instanceof DurationThresholdFilterSettings && initialSettings.getType() == type) {
                DurationThresholdFilterSettings settings = (DurationThresholdFilterSettings) initialSettings;
                jTextField.setText(ToStrings.duration.print(settings.getThresholdDuration().toPeriod().normalizedStandard()));
                jToggleButton.setSelected(settings.isInverted());
            } else {
                jToggleButton.setSelected(false);
            }
            return jTextField;
        }), graphPanel);

        panel.setTitle(title);

        SwingWorker<ChartPanel, Void> sw = new SwingWorker<ChartPanel, Void>() {
            @Override
            protected ChartPanel doInBackground() throws Exception {
                return chartPanelSupplier.get();
            }

            @Override
            protected void done() {
                try {
                    ChartPanel chartPanel = get();
                    graphPanel.removeAll();
                    graphPanel.add(chartPanel, BorderLayout.CENTER);
                    graphPanel.revalidate();
                    graphPanel.repaint();
                    panel.revalidate();
                    panel.repaint();
                } catch (InterruptedException | ExecutionException e) {
                    e.printStackTrace();
                }
            }
        };

        sw.execute();

        return panel;
    }

    public static DialogPanel<CaseIdFilterSettings> createCaseIdFilter(FilterSettings initialSettings) {
        DialogPanel<CaseIdFilterSettings> panel = createIncExcString("case ids to in/exclude", "include", "exclude", (String s, Boolean inverted) -> {
            String trim = s.replace("\n", ",");
            String[] split = trim.split(",");
            try {
                Set<Integer> collect = Arrays.stream(split).map(Integer::parseInt).collect(Collectors.toSet());
                return new CaseIdFilterSettings(collect, inverted);
            } catch (Exception ex) { // hopefully catches the NumberFormatException that might occur while mapping the stream
            }
            return null;
        }, ((jToggleButton) -> {
            JTextArea jta = new JTextArea(10, 20);
            jta.setLineWrap(true);
            if (initialSettings instanceof CaseIdFilterSettings && initialSettings.getType() == FilterType.CaseIds) {
                CaseIdFilterSettings settings = (CaseIdFilterSettings) initialSettings;
                jta.append(settings.getIdListString());
                jToggleButton.setSelected(settings.isInverted());
            } else {
                jToggleButton.setSelected(false);
            }
            return jta;
        }), null);
        panel.setTitle("Case Id Filter Settings");

        return panel;
    }

    private static <T1, T2 extends JTextComponent> DialogPanel<T1> createIncExcString(String labelString, String includeString, String excludeString, BiFunction<String, Boolean, T1> out, Function<JToggleButton, T2> inp, JPanel extra) {
        DialogPanel<T1> panel = new DialogPanel<T1>();
        if (extra != null) {
            GridBagConstraints c = panel.getGridBagConstraints();
            c.ipadx = 10;
            c.ipady = 10;

            panel.addComponent(extra);
            panel.newRow();
        }

        JLabel jLabel = SlickerFactory.instance().createLabel(labelString);

        JRadioButton include = SlickerFactory.instance().createRadioButton(includeString);
        JRadioButton exclude = SlickerFactory.instance().createRadioButton(excludeString);
        ButtonGroup bg = new ButtonGroup();
        bg.add(include);
        bg.add(exclude);
        JPanel buttons = new JPanel();
        buttons.add(include);
        buttons.add(exclude);
        include.setSelected(true);

        T2 apply = inp.apply(exclude);
        apply.addAncestorListener(new RequestFocusListener());
        panel.addComponent(jLabel);
        panel.addComponent(apply);
        panel.addComponent(buttons);

        panel.setResultExtractor(() -> out.apply(apply.getText(), exclude.isSelected()));

        return panel;
    }

    public static DialogPanel<PercentileClassificationSettings> createDurationPercentileClassification(ClassificationSettings initialSettings, FilterSettings filterSettings, ExtraComputationsController extraComputationsController) {
        return createPercentileGraphClassification(initialSettings, ClassificationType.CaseDurationPercentile, () -> {
            CaseIdFilter filter = extraComputationsController.getFilter(filterSettings);
            return extraComputationsController.getDurationHistogram(filter);
        }, DURATION_HISTOGRAM_WIDTH, DURATION_HISTOGRAM_HEIGHT, "Duration Classification Settings", "[%] of shortest cases to split at");
    }

    public static DialogPanel<PercentileClassificationSettings> createVariantPercentileClassification(ClassificationSettings initialSettings, FilterSettings filterSettings, ExtraComputationsController extraComputationsController) {
        return createPercentileGraphClassification(initialSettings, ClassificationType.VariantPercentile, () -> {
            CaseIdFilter filter = extraComputationsController.getFilter(filterSettings);
            return extraComputationsController.getVariantBarChart(filter);
        }, VARIANT_BARCHART_WIDTH, VARIANT_BARCHART_HEIGHT, "Variant Classification Settings", "[%] of most frequent cases to split at");
    }

    public static DialogPanel<PercentileClassificationSettings> createPercentileGraphClassification(ClassificationSettings initialSettings, ClassificationType type, Supplier<ChartPanel> chartPanelSupplier, int graphWidth, int graphHeight, String title, String labelString) {
        return createStringGraph(chartPanelSupplier, graphWidth, graphHeight, (String s) -> {
            try {
                double v = Double.parseDouble(s) / 100;
                return new PercentileClassificationSettings(type, v);
            } catch (NumberFormatException e) {
            }
            return null;
        }, () -> {
            JTextField jTextField = new JTextField("50");
            if (initialSettings instanceof PercentileClassificationSettings && initialSettings.getType() == type) {
                PercentileClassificationSettings settings = (PercentileClassificationSettings) initialSettings;
                jTextField.setText(String.valueOf(settings.getTopPercentage() * 100));
            }
            return jTextField;
        }, title, labelString);
    }

    public static DialogPanel<DurationThresholdClassificationSettings> createThresholdGraphClassification(ClassificationSettings initialSettings, FilterSettings filterSettings, ExtraComputationsController extraComputationsController) {
        return createStringGraph(() -> {
            CaseIdFilter filter = extraComputationsController.getFilter(filterSettings);
            return extraComputationsController.getDurationHistogram(filter);
        }, DURATION_HISTOGRAM_WIDTH, DURATION_HISTOGRAM_HEIGHT, s -> {
            try {
                Period period = Period.parse(s, ToStrings.duration);
                Duration duration = period.normalizedStandard().toDurationFrom(new Instant(0));
                return new DurationThresholdClassificationSettings(duration);
            } catch (IllegalArgumentException ex) {
            }
            return null;
        }, () -> {
            JTextField jTextField = new JTextField("");
            if (initialSettings instanceof DurationThresholdClassificationSettings && initialSettings.getType() == ClassificationType.CaseDurationThreshold) {
                DurationThresholdClassificationSettings settings = (DurationThresholdClassificationSettings) initialSettings;
                jTextField.setText(ToStrings.duration.print(settings.getThresholdDuration().toPeriod().normalizedStandard()));
            }
            return jTextField;
        }, "Duration Classification Settings", "Duration to split at [ex. 1w2d3h4m5s]");
    }

    public static DialogPanel<DurationClassesClassificationSettings> createClassesGraphClassification(ClassificationSettings initialSettings, FilterSettings filterSettings, ExtraComputationsController extraComputationsController) {
        return createStringGraph(() -> {
            CaseIdFilter filter = extraComputationsController.getFilter(filterSettings);
            return extraComputationsController.getDurationHistogram(filter);
        }, DURATION_HISTOGRAM_WIDTH, DURATION_HISTOGRAM_HEIGHT, s -> {
            try {
                return new DurationClassesClassificationSettings(Integer.parseInt(s));
            } catch (NumberFormatException ex) {
            }
            return null;
        }, () -> {
            JTextField jTextField = new JTextField("4");
            if (initialSettings instanceof DurationClassesClassificationSettings && initialSettings.getType() == ClassificationType.CaseDurationClasses) {
                DurationClassesClassificationSettings settings = (DurationClassesClassificationSettings) initialSettings;
                jTextField.setText(String.valueOf(settings.getClasses()));
            }
            return jTextField;
        }, "Duration Classification Settings", "Splits into n duration classes");
    }

    public static <T1, T2 extends JTextComponent> DialogPanel<T1> createStringGraph(Supplier<ChartPanel> chartPanelSupplier, int graphWidth, int graphHeight, Function<String, T1> out, Supplier<T2> inp, String title, String labelString) {
        JPanel graphPanel = new JPanel(new BorderLayout());
        graphPanel.add(new JLabel(SettingsPanel.loadingIcon), BorderLayout.CENTER);
        graphPanel.setPreferredSize(new Dimension(graphWidth, graphHeight));

        DialogPanel<T1> panel = createString(labelString, out, inp, graphPanel);

        panel.setTitle(title);

        SwingWorker<ChartPanel, Void> sw = new SwingWorker<ChartPanel, Void>() {
            @Override
            protected ChartPanel doInBackground() throws Exception {
                return chartPanelSupplier.get();
            }

            @Override
            protected void done() {
                try {
                    ChartPanel chartPanel = get();
                    graphPanel.removeAll();
                    graphPanel.add(chartPanel, BorderLayout.CENTER);
                    graphPanel.revalidate();
                    graphPanel.repaint();
                    panel.revalidate();
                    panel.repaint();
                } catch (InterruptedException | ExecutionException e) {
                    e.printStackTrace();
                }
            }
        };

        sw.execute();

        return panel;
    }

    private static <T1, T2 extends JTextComponent> DialogPanel<T1> createString(String labelString, Function<String, T1> out, Supplier<T2> inp, JPanel extra) {
        DialogPanel<T1> panel = new DialogPanel<T1>();
        if (extra != null) {
            GridBagConstraints c = panel.getGridBagConstraints();
            c.ipadx = 10;
            c.ipady = 10;
            panel.addComponent(extra);
            panel.newRow();
        }

        JLabel jLabel = SlickerFactory.instance().createLabel(labelString);

        T2 apply = inp.get();
        apply.addAncestorListener(new RequestFocusListener());

        panel.addComponent(jLabel);
        panel.addComponent(apply);

        panel.setResultExtractor(() -> out.apply(apply.getText()));

        return panel;
    }

    public GridBagConstraints getGridBagConstraints() {
        return c;
    }

    public void setGridBagConstraints(GridBagConstraints c) {
        this.c = c;
    }

    public void addComponent(JComponent jComponent) {
        addComponentToRow(jComponent);
        newRow();
    }

    public void addComponentToRow(JComponent jComponent) {
        add(jComponent, c);
        c.gridx++;
    }

    public void addComponent(JComponent jComponent, int gridwidth) {
        addComponentToRow(jComponent, gridwidth);
        newRow();
    }

    public void addComponentToRow(JComponent jComponent, int gridwidth) {
        c.gridwidth = gridwidth;
        add(jComponent, c);
        c.gridwidth = 1;
        c.gridx++;
    }

    public void newRow() {
        c.gridy++;
        c.gridx = 0;
    }

    public void setResultExtractor(Supplier<T> resultExtractor) {
        this.resultExtractor = resultExtractor;
    }

    public T getResult() {
        return resultExtractor.get();
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }
}
